from os.path import join
from collections import defaultdict

from django.conf import settings
from django.db import models
from django.apps import apps
from django.template.defaultfilters import pluralize
from django.core.management.base import BaseCommand

from django_broom.utils import get_files, remove_file


class Command(BaseCommand):

    def add_arguments(self, parser):

        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Remove all unused media files without confirmation'
        )

    def handle(self, *args, **options):
        """
        Locates and removes all unused media files.

        Action must be confirmed unless `--force` option is used (useful
        when setting up a cron job).
        """

        self.stdout.write('Locating files...')

        files = defaultdict(lambda: [])

        for model in apps.get_models():
            for field in model._meta.get_fields():
                if isinstance(field, models.FileField):
                    directory = field.upload_to

                    if directory not in files:
                        files[directory] = get_files(directory)

                    for file_name in files[directory]:
                        kwargs = {field.name: join(directory, file_name)}

                        if model.objects.filter(**kwargs).exists():
                            files[directory].remove(file_name)

        total = sum(len(directory) for directory in files.itervalues())

        if total > 0:
            question = 'Remove %s unused file%s?' % (total, pluralize(total))
            answer = None

            while not options['force'] and answer not in ['y', 'n']:
                if answer is not None:
                    self.stdout.write('Invalid answer. Try again!')

                answer = raw_input(question + ' (y/n): ').lower().strip()

            if options['force'] or answer == 'y':
                for directory, files in files.iteritems():
                    directory = join(settings.MEDIA_ROOT, directory)

                    for file_name in files:
                        path_to_file = join(directory, file_name)

                        if remove_file(path_to_file):
                            self.stdout.write(
                                'File %s removed.' % path_to_file
                            )

                self.stdout.write('Done.')
            else:
                self.stderr.write('Aborted.')
        else:
            self.stdout.write('No unused files found.')
