from os import listdir, rmdir, remove
from os.path import isdir, isfile, join

from mock import patch

from django.conf import settings
from django.core.management import call_command
from django.core.files import File
from django.test import TestCase
from django.test.utils import captured_output

from django_broom.tests.app.models import FirstModel, SecondModel


class CleanMediaFilesTest(TestCase):

    def setUp(self):

        self.image = File(open(join(settings.MEDIA_ROOT, 'image.png')))

        self.first_object = FirstModel.objects.create(image=self.image)
        self.second_object = SecondModel.objects.create(image=self.image)

        self.message = 'Locating files...\n'

    def tearDown(self):

        directory = join(settings.MEDIA_ROOT, 'images')

        if isdir(directory):
            for file_name in listdir(directory):
                path_to_file = (join(directory, file_name))

                if isfile(path_to_file):
                    remove(path_to_file)

            rmdir(directory)

    def test_command_when_no_unused_media_files(self):
        """ Informs when no unused files found """

        self.message += 'No unused files found.\n'

        with captured_output('stdout') as stdout, \
                captured_output('stderr') as stderr:

            call_command('cleanmediafiles')

            assert stdout.getvalue() == self.message
            assert stderr.getvalue() == ''

    def test_command_when_forcing(self):
        """ Removes without confirmation and informs about files removed """

        self.message += 'File %s removed.\n' % self.first_object.image.path
        self.first_object.image = self.image
        self.first_object.save()

        self.message += 'File %s removed.\n' % self.second_object.image.path
        self.second_object.image = self.image
        self.second_object.save()

        self.message += 'Done.\n'

        with captured_output('stdout') as stdout, \
                captured_output('stderr') as stderr:

            call_command('cleanmediafiles', force=True)

            assert stdout.getvalue() == self.message
            assert stderr.getvalue() == ''

    def test_command_when_aborting(self):
        """ Aborts when not confirmed and informs about it """

        self.first_object.image = self.image
        self.first_object.save()

        self.second_object.image = self.image
        self.second_object.save()

        with patch('__builtin__.raw_input', return_value='n'):
            with captured_output('stdout') as stdout, \
                    captured_output('stderr') as stderr:

                call_command('cleanmediafiles')

                assert stdout.getvalue() == self.message
                assert stderr.getvalue() == 'Aborted.\n'

    def test_command(self):
        """ Removes when confirmed and informs about files removed """

        self.message += 'File %s removed.\n' % self.first_object.image.path
        self.first_object.image = self.image
        self.first_object.save()

        self.message += 'File %s removed.\n' % self.second_object.image.path
        self.second_object.image = self.image
        self.second_object.save()

        self.message += 'Done.\n'

        with patch('__builtin__.raw_input', return_value='y'):
            with captured_output('stdout') as stdout, \
                    captured_output('stderr') as stderr:

                call_command('cleanmediafiles')

                assert stdout.getvalue() == self.message
                assert stderr.getvalue() == ''
